"use strict";

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});

const universalify_1 = require("universalify");

const fs_1 = __importDefault(require("fs"));

const rcs_core_1 = __importDefault(require("rcs-core"));

const path_1 = __importDefault(require("path"));

const glob_1 = __importDefault(require("glob"));

const async_1 = require("async");

const assert_1 = __importDefault(require("assert"));

const save_1 = __importDefault(require("../helper/save"));

const replaceData_1 = __importDefault(require("./replaceData"));

const defaults_1 = __importDefault(require("./defaults"));

const Config_1 = __importDefault(require("../Config"));

const {
  fileExt,
  availableTypes,
  optionsDefault
} = defaults_1.default;

async function rcsProcess(type, pathString, opts = {}) {
  var _a;

  const options = Object.assign(Object.assign({}, optionsDefault), opts);
  const shouldOptimize = (_a = options.optimize, _a !== null && _a !== void 0 ? _a : true);
  let globString;
  assert_1.default(availableTypes.includes(type), `type must be one of the following: ${availableTypes}`);

  if (Array.isArray(pathString)) {
    globString = pathString.length > 1 ? `{${pathString.join(',')}}` : pathString[0];
  } else {
    globString = pathString;
  }

  const cwd = options.cwd || process.cwd();
  const filesArray = await new Promise((res, rej) => glob_1.default(globString, {
    cwd,
    ignore: Config_1.default.getInstance().ignorePatterns
  }, (error, allFiles) => error ? rej(error) : res(allFiles))); // fail if nothing is found

  if (filesArray.length <= 0) {
    throw new Error('No files found');
  } // sort in case of 'auto'


  const cssHtmlFiles = filesArray.filter(file => fileExt.css.includes(path_1.default.extname(file)) || fileExt.html.includes(path_1.default.extname(file)));
  const fillLibraryFiles = type === 'auto' ? cssHtmlFiles : filesArray; // call in series
  // not all selectors are stored, maybe some selectors are duplicated in different files

  await new Promise((res, rej) => async_1.eachSeries(fillLibraryFiles, (filePath, asyncCb) => {
    // skip if it is not meant to fill the library
    if (type !== 'auto' && type !== 'css' && type !== 'html') {
      return asyncCb();
    }

    return fs_1.default.readFile(path_1.default.join(cwd, filePath), (errReadFile, bufferData) => {
      if (errReadFile) {
        return asyncCb(errReadFile);
      } // add here again so typescript gets the correct option types


      if (type !== 'auto' && type !== 'css' && type !== 'html') {
        return undefined;
      }

      const isHtml = fileExt.html.includes(path_1.default.extname(filePath));
      rcs_core_1.default.fillLibraries(bufferData.toString(), {
        prefix: options.prefix,
        suffix: options.suffix,
        replaceKeyframes: options.replaceKeyframes,
        preventRandomName: options.preventRandomName,
        ignoreAttributeSelectors: options.ignoreAttributeSelectors,
        ignoreCssVariables: options.ignoreCssVariables,
        codeType: isHtml ? 'html' : 'css'
      });
      return asyncCb();
    });
  }, err => err ? rej(err) : res()));

  if (shouldOptimize) {
    rcs_core_1.default.optimize();
  }

  await new Promise((res, rej) => // now all files can be renamed and saved
  // can be fired parallel
  // all selectors are collected
  // ⚡️ speed it up with async/each ⚡️
  async_1.each(filesArray, (filePath, asyncEachCb) => {
    fs_1.default.readFile(path_1.default.join(cwd, filePath), 'utf8', (err, bufferData) => {
      let data;
      let shouldOverwrite = options.overwrite;

      if (err) {
        return asyncEachCb(err);
      }

      try {
        data = replaceData_1.default(type, filePath, bufferData, options);
      } catch (e) {
        return asyncEachCb(e);
      }

      const joinedPath = path_1.default.join(options.newPath || cwd, filePath);

      if (!options.overwrite) {
        shouldOverwrite = joinedPath !== path_1.default.join(cwd, filePath);
      }

      return save_1.default(joinedPath, data, {
        overwrite: shouldOverwrite
      }, errSave => {
        if (errSave) {
          return asyncEachCb(errSave);
        }

        return asyncEachCb(null);
      });
    });
  }, errProcess => {
    rcs_core_1.default.warnings.warn();

    if (errProcess) {
      rej(errProcess);
    } else {
      res();
    }
  }));
} // /process


exports.default = universalify_1.fromPromise(rcsProcess);
module.exports = exports.default;